﻿// ECMAScript 5 strict mode
"use strict";

assert2(cr, "cr namespace not created");
assert2(cr.plugins_, "cr.plugins_ not created");

/////////////////////////////////////
// Plugin class
// *** CHANGE THE PLUGIN ID HERE *** - must match the "id" property in edittime.js
//          vvvvvvvv
cr.plugins_.mcPasteSprite = function(runtime)
{
	this.runtime = runtime;
};

(function ()
{
	/////////////////////////////////////
	// *** CHANGE THE PLUGIN ID HERE *** - must match the "id" property in edittime.js
	//                            vvvvvvvv
	var pluginProto = cr.plugins_.mcPasteSprite.prototype;
		
	/////////////////////////////////////
	// Object type class
	pluginProto.Type = function(plugin)
	{
		this.plugin = plugin;
		this.runtime = plugin.runtime;
	};

	var typeProto = pluginProto.Type.prototype;

	// called on startup for each object type
	typeProto.onCreate = function()
	{
	};

	/////////////////////////////////////
	// Instance class
	pluginProto.Instance = function(type)
	{
		this.type = type;
		this.runtime = type.runtime;
		
		// any other properties you need, e.g...
		// this.myValue = 0;
	};
	
	var instanceProto = pluginProto.Instance.prototype;

	// called whenever an instance is created
	instanceProto.onCreate = function()
	{
		// note the object is sealed after this call; ensure any properties you'll ever need are set on the object
		// e.g...
		// this.myValue = 0;
		this.canvasQR = document.createElement('canvas');
		this.ctxQR = this.canvasQR.getContext('2d');
		
		this.canvasQR.width = 512;
		this.canvasQR.height = 512;
		
		//this.canvasQR = null;
		//this.ctxQR = null;
		
		
		this.rcTex = new cr.rect(0, 0, 1, 1);
		this.update_tex = false;
		
		this.pasteSprite = null;
		
		// For WebGL rendering
		this.mycanvas = null;
		this.myctx = null;
		this.mytex = null;
		
		this.Axx = 0;
		this.Ayy = 0;
		this.Awidth = 0;
		this.Aheight = 0;
		this.Aax = 0;
		this.Aay = 0;
		this.Adw = 0;
		this.Adh = 0;
	};
	
	// called whenever an instance is destroyed
	// note the runtime may keep the object after this call for recycling; be sure
	// to release/recycle/reset any references to other objects in this function.
	instanceProto.onDestroy = function ()
	{
		this.myctx = null;
		this.mycanvas = null;
		this.canvasQR = null;
		this.ctxQR = null;
		if (this.runtime.glwrap && this.mytex)
			this.runtime.glwrap.deleteTexture(this.mytex);
		
		this.mytex = null;
	};
	
	// called when saving the full state of the game
	instanceProto.saveToJSON = function ()
	{
		// return a Javascript object containing information about your object's state
		// note you MUST use double-quote syntax (e.g. "property": value) to prevent
		// Closure Compiler renaming and breaking the save format
		return {
			// e.g.
			//"myValue": this.myValue
		};
	};
	
	// called when loading the full state of the game
	instanceProto.loadFromJSON = function (o)
	{
		// load from the state previously saved by saveToJSON
		// 'o' provides the same object that you saved, e.g.
		// this.myValue = o["myValue"];
		// note you MUST use double-quote syntax (e.g. o["property"]) to prevent
		// Closure Compiler renaming and breaking the save format
	};
	
	// only called if a layout object - draw to a canvas 2D context
	instanceProto.draw = function(ctx)
	{
		ctx.globalAlpha = this.opacity;
		
		// DC doesn't return globalCompositeOperation correctly
		if (this.runtime.isDirectCanvas)
		{
			if (ctx.curCompOp !== this.compositeOp)
			{
				ctx.globalCompositeOperation = this.compositeOp;
				ctx.curCompOp = this.compositeOp;
			}
		}
		else
			ctx.globalCompositeOperation = this.compositeOp;
			
		// The current animation frame to draw
		/*var cur_frame = this.cur_animation.frames[this.cur_frame];
		var cur_image = cur_frame.texture_img;*/
		
		var myx;
		var myy;
		
		// Object not rotated: can draw without transformation.
		if (this.angle === 0 && this.width >= 0 && this.height >= 0)
		{
			myx = this.x - (this.hotspotX * this.width);
			myy = this.y - (this.hotspotY * this.height);
			
			if (this.runtime.pixel_rounding)
			{
				myx = Math.round(myx);
				myy = Math.round(myy);
			}
			
			//ctx.drawImage(cur_image,
			ctx.drawImage(this.canvasQR,
						  myx,
						  myy,
						  this.width,
						  this.height);
		}
		else
		{
			// Only pixel round the x/y position, otherwise objects don't rotate smoothly
			myx = this.x;
			myy = this.y;
			
			if (this.runtime.pixel_rounding)
			{
				myx = Math.round(myx);
				myy = Math.round(myy);
			}
			
			// Angle applied; we need to transform the canvas.  Save state.
			ctx.save();
			
			var widthfactor = this.width > 0 ? 1 : -1;
			var heightfactor = this.height > 0 ? 1 : -1;
			
			// Translate to object's position, then rotate by its angle.
			ctx.translate(myx, myy);
			ctx.scale(widthfactor, heightfactor);
			ctx.rotate(this.angle * widthfactor * heightfactor);
			
			// Draw the object; canvas origin is at hot spot.
			//ctx.drawImage(cur_image,
			ctx.drawImage(this.canvasQR,
						  0 - (this.hotspotX * Math.abs(this.width)),
						  0 - (this.hotspotY * Math.abs(this.height)),
						  Math.abs(this.width),
						  Math.abs(this.height));
			
			// Restore previous state.
			ctx.restore();
		}
	
	};
	
	// only called if a layout object in WebGL mode - draw to the WebGL context
	// 'glw' is not a WebGL context, it's a wrapper - you can find its methods in GLWrap.js in the install
	// directory or just copy what other plugins do.
	instanceProto.drawGL = function (glw)
	{
		glw.setBlend(this.srcBlend, this.destBlend);
        if (this.update_tex)
        {
            if (this.tex)
                glw.deleteTexture(this.tex);
            if (this.runtime.pixel_rounding) {this.tex=glw.loadTexture(this.canvasQR, false, this.runtime.linearSampling);}
			else {this.tex=glw.loadTexture(this.canvasQR, false, this.runtime.pointSampling);}
            this.update_tex = false;
        }
		glw.setTexture(this.tex);
		glw.setOpacity(this.opacity);

		var q = this.bquad;
		
		if (this.runtime.pixel_rounding)
		{
			var ox = Math.round(this.x) - this.x;
			var oy = Math.round(this.y) - this.y;
			
			glw.quad(q.tlx + ox, q.tly + oy, q.trx + ox, q.try_ + oy, q.brx + ox, q.bry + oy, q.blx + ox, q.bly + oy);
		}
		else
			glw.quad(q.tlx, q.tly, q.trx, q.try_, q.brx, q.bry, q.blx, q.bly);
	};
	
	// The comments around these functions ensure they are removed when exporting, since the
	// debugger code is no longer relevant after publishing.
	/**BEGIN-PREVIEWONLY**/
	instanceProto.getDebuggerValues = function (propsections)
	{
		// Append to propsections any debugger sections you want to appear.
		// Each section is an object with two members: "title" and "properties".
		// "properties" is an array of individual debugger properties to display
		// with their name and value, and some other optional settings.
		propsections.push({
			"title": "My debugger section",
			"properties": [
				// Each property entry can use the following values:
				// "name" (required): name of the property (must be unique within this section)
				// "value" (required): a boolean, number or string for the value
				// "html" (optional, default false): set to true to interpret the name and value
				//									 as HTML strings rather than simple plain text
				// "readonly" (optional, default false): set to true to disable editing the property
				
				// Example:
				// {"name": "My property", "value": this.myValue}
			]
		});
	};
	
	instanceProto.onDebugValueEdited = function (header, name, value)
	{
		// Called when a non-readonly property has been edited in the debugger. Usually you only
		// will need 'name' (the property name) and 'value', but you can also use 'header' (the
		// header title for the section) to distinguish properties with the same name.
		if (name === "My property")
			this.myProperty = value;
	};
	/**END-PREVIEWONLY**/

	//////////////////////////////////////
	// Conditions
	function Cnds() {};

	// the example condition
	Cnds.prototype.MyCondition = function (myparam)
	{
		// return true if number is positive
		return myparam >= 0;
	};
	
	// ... other conditions here ...
	
	pluginProto.cnds = new Cnds();
	
	//////////////////////////////////////
	// Actions
	function Acts() {};


	Acts.prototype.setSize = function (xx,yy)
	{
		this.canvasQR.setAttribute('width',xx);
		this.canvasQR.setAttribute('height',yy);
	};
	

	
	Acts.prototype.pasteSpriteFromJSON = function (obj,istart,iistart,dw,dh,width,height,sizew,sizeh,ar,sizegfx,curani,curins)
	{
		var o;

		try {
			o = JSON.parse(ar);
		}
		catch(e) { return; }
		
		if(typeof obj.animations[curani].frames[obj.instances[curins].cur_frame].texture_img == "undefined") {return;}
		var source_image = obj.animations[curani].frames[obj.instances[curins].cur_frame].texture_img;
		
		var arr = o;
		
		var curWx = 0;
		var curHy = 0;
	
		
		for(var i=0;i<sizew;i++) {
			
			for(var ii=0;ii<sizeh;ii++) {
				
				var value = arr[i][ii][0];
				var curmax_x = 48; // 178 / 16 = 48
				
				
				var y = Math.floor(value/curmax_x)*sizegfx;
				
				//var res_x = Math.ceil(value/curmax_x);
				var x = (value % curmax_x)*sizegfx;
				
				
				//this.ctxQR.drawImage(source_image, ((curWx*sizegfx)+(arr[i][ii][0]*sizegfx)), (curHy*sizegfx), width, height, i*sizegfx, ii*sizegfx, dw, dh);
				this.ctxQR.drawImage(source_image, x, y, width, height, i*sizegfx, ii*sizegfx, dw, dh);
				
				
				
				
				
				this.pasteSprite = source_image;
				
			}
			
			curWx++;
			if(curWx>48) {curHy++; curWx=0;}
			
		}
		
	}
	
	Acts.prototype.convertToGray = function (obj,xx,yy,dw,dh,ax,ay,width,height,curani,curins)
	{	
		if(typeof obj.animations[curani].frames[obj.instances[curins].cur_frame].texture_img == "undefined") {return;}
		var source_image = obj.animations[curani].frames[obj.instances[curins].cur_frame].texture_img;
		
		this.ctxQR.drawImage(source_image, xx, yy, width, height, ax, ay, dw, dh);
		
		var imageData = this.ctxQR.getImageData(xx,yy,width,height);
		
		var data = imageData.data;
		
		for(var i=0;i<data.length;i+=4) {
			var brightness = 0.34 * data[i]+0.5*data[i+1]+0.16*data[i+2];
			//rot
			data[i]=brightness;
			//grün
			data[i+1]=brightness;
			//blau
			data[i+2]=brightness;
		}
		
		this.ctxQR.putImageData(imageData,xx,yy);
		
		this.Axx = xx;
		this.Ayy = yy;
		this.Awidth = width;
		this.Aheight = height;
		this.Aax = ax;
		this.Aay = ay;
		this.Adw = dw;
		this.Adh = dh;
	};
	
	Acts.prototype.pasteSprite = function (obj,xx,yy,dw,dh,ax,ay,width,height,curani,curins)
	{
		// alert the message
		//alert(myparam);
		if(typeof obj.animations[curani].frames[obj.instances[curins].cur_frame].texture_img == "undefined") {return;}
		var source_image = obj.animations[curani].frames[obj.instances[curins].cur_frame].texture_img;
		
		this.ctxQR.drawImage(source_image, xx, yy, width, height, ax, ay, dw, dh);
		
		this.pasteSprite = source_image;
		
		this.Axx = xx;
		this.Ayy = yy;
		this.Awidth = width;
		this.Aheight = height;
		this.Aax = ax;
		this.Aay = ay;
		this.Adw = dw;
		this.Adh = dh;

	};
	
	Acts.prototype.redrawspr = function ()
	{
		this.update_tex = true;
		this.runtime.redraw = true;
	};
	
	// ... other actions here ...
	
	pluginProto.acts = new Acts();
	
	//////////////////////////////////////
	// Expressions
	function Exps() {};
	
	// the example expression
	Exps.prototype.toBase64 = function (ret)	// 'ret' must always be the first parameter - always return the expression's result through it!
	{

		this.dataURL = this.canvasQR.toDataURL("image/png");
		this.dataURL.replace(/^data:image\/(png|jpg);base64,/, "");

		ret.set_string(this.dataURL);
		
	};
	
	Exps.prototype.toBase64AsJSON = function (ret)	// 'ret' must always be the first parameter - always return the expression's result through it!
	{

		this.dataURL = this.canvasQR.toDataURL("image/png");
		this.dataURL.replace(/^data:image\/(png|jpg);base64,/, "");

		var val_tmp = '{';
		val_tmp+='"img":"';
		val_tmp+=this.dataURL;
		val_tmp+='"}';
		
		ret.set_string(val_tmp);
		
	};
	
	
	
	Exps.prototype.toBase64WReplace = function (ret)	// 'ret' must always be the first parameter - always return the expression's result through it!
	{
		this.dataURL = this.canvasQR.toDataURL("image/png");
		ret.set_string(this.dataURL);
	};
	
	
	
	// ... other expressions here ...
	
	pluginProto.exps = new Exps();

}());